/*
 * *******************************************************************************************
 *  * Created By Debojyoti Singha
 *  * Copyright (c) 2022.
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  * http://www.apache.org/licenses/LICENSE-2.0
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *  * Contact Email: swingtechnologies452@gmail.com
 * ******************************************************************************************
 */

import 'package:dex/dex.dart';
import 'package:flutter/material.dart';

mixin DxViewBuilderListener<C, S> {
  void onStart();

  void onPause();

  void onStop();

  void onResume();

  C initiateController();

  Widget onBuild(BuildContext context, S? state);

  Widget onLoading(BuildContext context);
}

abstract class DxViewBuilder<C extends DxController<S>, S extends DxState>
    extends State with DxViewBuilderListener<C, S> {
  late C _controller;

  C get controller => _controller;

  @override
  void initState() {
    super.initState();
    _controller = initiateController();
  }

  @override
  Widget build(BuildContext context) {
    return StreamBuilder<S>(
      stream: _controller.observe(),
      builder: (context, snapshot) {
        return onBuild(context, snapshot.data);
      },
    );
  }
}
