/*
 * *******************************************************************************************
 *  * Created By Debojyoti Singha
 *  * Copyright (c) 2022.
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  * http://www.apache.org/licenses/LICENSE-2.0
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *  * Contact Email: swingtechnologies452@gmail.com
 * ******************************************************************************************
 */

import 'package:flutter/material.dart';

import '../builder/dx_widget_builder.dart';

abstract class DxParserBuilder {
  Map<String, dynamic> fromJson();

  String toJson(Map<String, dynamic> map);

  List<Widget> toChildren();

  List<DxWidgetBuilder> children = <DxWidgetBuilder>[];

  bool isChildrenAvailable() => children.isNotEmpty;

  DxWidgetBuilder? child() => children.isNotEmpty ? children.first : null;
}

class DxParser extends DxParserBuilder {
  Map<String, dynamic> dataMap = {};
  String data = "";
  List<Widget> _children = [];

  DxParser(String jsonData) {
    this.data = jsonData;
  }

  @override
  List<Widget> toChildren() {
    var widget = data.replaceAll("\"", "");

    // var text = "Text('Hi I am deb 2')".replaceAll("\"", "") as Widget;

    // logger.d(widget);
    _children.add(Container(
      child: Center(
        child: getWidget(widget, {}),
      ),
    ));
    return _children;
  }

  bool checkIfBodyPresent(Iterable<String> count) {
    return count.contains('body');
  }

  Widget getWidget(String widgetName, Map<String, dynamic> mapData) {
    if (widgetName is Container) {
      return Container();
    } else if (widgetName is Text) {
      return Text(mapData['text']);
    } else {
      return SizedBox();
    }

    /* switch (widgetName) {
      case 'scaffold':
        {
          return Scaffold();
        }

      case 'column':
        {
          return Column();
        }

      case 'text':
        {
          return Text('');
        }

      default:
        {
          return Container();
        }
    }*/
  }

  @override
  String toJson(Map<String, dynamic> map) {
    return map.toString();
  }

  @override
  Map<String, dynamic> fromJson() {
    return dataMap;
  }
}
