/*
 * *******************************************************************************************
 *  * Created By Debojyoti Singha
 *  * Copyright (c) 2022.
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  * http://www.apache.org/licenses/LICENSE-2.0
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *  * Contact Email: swingtechnologies452@gmail.com
 * ******************************************************************************************
 */

import 'dart:async';

import 'package:dex/src/state_manager/dx_state.dart';
import 'package:rxdart/rxdart.dart';

mixin DxStateListener {
  void onInitialize();

  void onDeInitialize();

  List<dynamic> onVariables();
}

abstract class DxController<S extends DxState> implements DxStateListener {
  List<dynamic> get _getVariableList => onVariables();
  late PublishSubject<S> _controller;
  late S _dxState;

  S get dxState => _dxState;

  DxController(S dxState) {
    this._dxState = dxState;
    _controller = PublishSubject<S>();
    onInitialize();
  }

  void _declareType() {
    _getVariableList.forEach((element) {
      _controller.sink.add(element);
    });
  }

  void update(S data) {
    _controller.sink.add(data);
  }

  Stream<S> observe() {
    return _controller.stream;
  }
}
