/*
 * *******************************************************************************************
 *  * Created By Debojyoti Singha
 *  * Copyright (c) 2022.
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  * http://www.apache.org/licenses/LICENSE-2.0
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *  * Contact Email: swingtechnologies452@gmail.com
 * ******************************************************************************************
 */

import 'package:dex/dex.dart';
import 'package:dex/src/builder/dx_widget_builder.dart';
import 'package:flutter/material.dart';

import '../extension/dx_margin.dart';

class DxContainer with DxWidgetBuilder {
  DxContainer({this.child});

  DxInsets? _dxMargin;
  DxInsets? _dxPadding;

  Color? _color;

  Widget? child;

  BoxDecoration? _boxDecoration;

  DxContainer setMargin(DxInsets dxInsets) {
    _dxMargin = dxInsets;
    return this;
  }

  DxContainer setPadding(DxInsets dxInsets) {
    _dxPadding = dxInsets;
    return this;
  }

  DxContainer setBackgroundColor(Color color) {
    this._color = color;
    return this;
  }

  DxContainer setDecoration(BoxDecoration decoration) {
    this._boxDecoration = decoration;
    return this;
  }

  DxContainer setChild(Widget child) {
    this.child = child;
    return this;
  }

  @override
  Widget create() {
    return Container(
      child: child,
      margin: _dxMargin,
      padding: _dxPadding,
      color: _boxDecoration == null ? _color : null,
      decoration: _boxDecoration,
    );
  }

  @override
  InkWell toButton(VoidCallback onClick) {
    return InkWell(
      child: create(),
      onTap: onClick,
    );
  }
}
